<?php

namespace Ems\Util;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Doctrine\ORM\EntityManagerInterface;
use Util\Environment;

class Utils {

    const ORDEM_ASC     = 'ASC';
    const ORDEM_DESC    = 'DESC';
    
    const SIM = 'S';
    const NAO = 'N';

    const DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION = 3;
    
    const DATA_OPERATION_INSERT = 'INSERT';
    const DATA_OPERATION_UPDATE = 'UPDATE';
    const DATA_OPERATION_DELETE = 'DELETE';

    const REMOVER_ESPECIAIS = array(
        "ª" => "", "º" => "", "à" => "a","á" => "a","ã" => "a","â" => "a","é" => "e","ê" => "e","ì" => "i","í" => "i",
        "ó" => "o", "õ" => "o","ô" => "o","ú" => "u","ü" => "u","ç" => "c","À" => "A","Á" => "A","Ã" => "A","Â" => "A",
        "É" => "E","Ê" => "E","Í" => "I","Ó" => "O","Õ" => "O","Ô" => "O","Ù" => "U","Ú" => "U","Ü" => "U","." => ""
    );

    const REMOVER_INVALIDOS = array(
        "ª" => "", "º" => "", "à" => "a","á" => "a","ã" => "a","â" => "a","é" => "e","ê" => "e","ì" => "i","í" => "i",
        "ó" => "o", "õ" => "o","ô" => "o","ú" => "u","ü" => "u","ç" => "c","À" => "A","Á" => "A","Ã" => "A","Â" => "A",
        "É" => "E","Ê" => "E","Í" => "I","Ó" => "O","Õ" => "O","Ô" => "O","Ù" => "U","Ú" => "U","Ü" => "U", " " => "_",
        "." => ""
    );
    
    const ORG_TEKNISA = 20;
    
    public static function getInstance(string $instanceName) {
        $instanceManager = InstanceManager::getInstance();
        return $instanceManager->getService($instanceName);
    }
    
    public static function getEntityManager(): ?EntityManagerInterface {
        return self::getInstance('entityManager');
    }
    
    public static function getParamters(): ?Environment {
        return self::getInstance('\Util\Environment');
    }

    public static function removeCaracteresEspeciais($str){
        return strtr($str, self::REMOVER_ESPECIAIS);
    }

    public static function removeCaracteresInvalidos($str){
        return strtr($str, self::REMOVER_INVALIDOS);
    }

    public static function fixName(
        string $name
    ): string {
        $name = trim($name);
        $name = self::removeCaracteresEspeciais($name);
        $name = strtoupper($name);

        return $name;
    }

    public static function getIdDiaSemana(string $date): string {
        $formattedDate = Utils::formatDateYMD($date);

        $dayOfWeek = date('w', strtotime($formattedDate));

        $idRelation = [
            '1000000',
            '0100000',
            '0010000',
            '0001000',
            '0000100',
            '0000010',
            '0000001',
        ];

        $idDiaSemana = $idRelation[$dayOfWeek];

        return $idDiaSemana;
    }

    public static function formatDateYMD(string $date): string {
        [$day, $month, $year] = explode('/', $date);
        $formattedDate = "{$year}-{$month}-{$day}";

        return $formattedDate;
    }

    public static function formatDateDMY(string $date): string {
        [$year, $month, $day] = explode('-', $date);
        $formattedDate = "{$day}/{$month}/{$year}";

        return $formattedDate;
    }

    public static function stringToDateTime(string $date): \DateTimeImmutable {
        $formattedDate = Utils::formatDateYMD($date);
        $dateTime = new \DateTimeImmutable($formattedDate);

        return $dateTime;
    }

    public static function addDaysToDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->add($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }

    public static function subDaysFromDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->sub($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }
    
    public static function areFloatsEqual($a, $b, $precision = self::DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION) {
        return abs(($a-$b)/$b) < pow(10, -1 * $precision);
    }

    public static function getSqlDateTimeString(
        string $date,
        string $time
    ): string {
        $dateTime = "$date $time:00";

        return $dateTime;
    }

    /**
     * Preenche os campos especificados da row com o valor escolhido caso não esteja definido.
     *
     * @param array $row A row a ser preenchida.
     * @param string[] $fields Os campos que, caso não estejam definidos, serão preenchidos com o valor especificado.
     * @param mixed $value O valor que preencherá os campos especificados.
     *
     * @return array A row enviada com os campos preenchidos.
     */
    public static function fillRowWithValueIfUndefined(
        array $row,
        array $fields,
        $value = null
    ): array {
        foreach ($fields as $field) {
            $row[$field] = $row[$field] ?? $value;
        }

        return $row;
    }

    /**
     * Remove os pontos e vírgulas da string.
     *
     * @param string $str A string a ter a pontuação removida.
     *
     * @return string A string passado com a pontuação removida.
     */
    public static function removePunctuation(
        string $str
    ): string {
        $search = ['.', ',', '-'];
        $str = str_replace($search, '', $str);

        return $str;
    }

    public static function currencyStringToNumber(
        string $value
    ) {
        $value = str_replace(['R', 'r', '.', '$'], '', $value);
        $value = str_replace(',', '.', $value);

        $valueNumber = floatval($value);

        return $valueNumber;
    }

    public static function getIdOcorrenciaDescription(
        string $idConfirmada
    ) {
        $relation = [
            'P' => 'Pendente',
            'A' => 'Aprovada',
            'R' => 'Rejeitada',
        ];

        $description = $relation[$idConfirmada] ?? false;

        return $description;
    }
    
    public static function decompoeNomePessoa($nmpessoa)
    {
        $arrayNmpessoa = explode(' ', $nmpessoa);
        $nomeMeio      = '';

        $primeiroNome = $arrayNmpessoa[0];
        for ($i = 1; $i < count($arrayNmpessoa) - 1; $i++) {
            $nomeMeio .= ' ' . $arrayNmpessoa[$i];
            $nomeMeio = trim($nomeMeio);
        }
        $ultimoNome = $arrayNmpessoa[count($arrayNmpessoa) - 1];
        $nomeMeio   = ($nomeMeio) ? : $ultimoNome;
        return array($primeiroNome, $nomeMeio, $ultimoNome);
    }
    
    public static function getPronomeTratamento($cdEstadoCivil,$idSexoPessoa) {
        if ($idSexoPessoa == 'M'){
            $pronomeTratamento = 'Sr.';
            return $pronomeTratamento;
        }
        else if ($idSexoPessoa == 'F' && $cdEstadoCivil == 'C'){
            $pronomeTratamento = 'Sra.';
            return $pronomeTratamento;
        }
        else {
            $pronomeTratamento = 'Srta.';
            return $pronomeTratamento;
        }
    }
    
    public static function sanitizeCPF(string $cpf): string {
        return str_replace(array('.','/','-'), '', $cpf);
    }
    
    public static function sanitizeUrl(string $url): string {
        return strtr(
            $url,
            [' ' => '%20']
        );
    }
}